@@ -1,26 +1,32 @@
+{-# LANGUAGE CPP              #-}
+{-# LANGUAGE FlexibleContexts #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Main where
 
-import Control.Applicative
-import Control.Monad   (when)
-import Data.Semigroup
+#if __GLASGOW_HASKELL__ < 710
+import           Control.Applicative
+#endif
 
-import System.Exit     (exitFailure)
+import           Control.Monad       (unless)
+import           Data.Semigroup
 
-import Test.QuickCheck
-import Text.Printf     (printf)
+import           System.Exit         (exitFailure)
 
-import Data.Active
-import Data.VectorSpace
-import Data.AffineSpace
+import           Test.QuickCheck
+import           Text.Printf         (printf)
+
+import           Data.Active
+
+import           Linear.Affine
+import           Linear.Vector
 
 main :: IO ()
 main = do
-    results <- mapM (\(s,t) -> printf "%-40s" s >> t) tests
-    when (not . all isSuccess $ results) $ exitFailure
+  results <- mapM (\(s,t) -> printf "%-40s" s >> t) tests
+  unless (all isSuccess results) exitFailure
   where
     isSuccess (Success{}) = True
-    isSuccess _ = False
+    isSuccess _           = False
     qc x = quickCheckWithResult (stdArgs { maxSuccess = 200 }) x
     tests = [ ("era/start",                   qc prop_era_start          )
             , ("era/end",                     qc prop_era_end            )
@@ -42,17 +48,19 @@ main = do
             , ("atTime/start",                qc prop_atTime_start       )
             , ("atTime/fun",                  qc prop_atTime_fun         )
             ]
+{-# ANN main ("HLint: ignore Eta reduce" :: String) #-}
+-- eta reducing qc breaks it
 
 instance Arbitrary Any where
   arbitrary = Any <$> arbitrary
 
-instance Arbitrary Time where
+instance (Arbitrary n, Fractional n, Real n) => Arbitrary (Time n) where
   arbitrary = fromRational <$> arbitrary
 
-instance CoArbitrary Time where
+instance (CoArbitrary n, Real n) => CoArbitrary (Time n) where
   coarbitrary t = coarbitrary (toRational t)
 
-instance Arbitrary Duration where
+instance (Arbitrary n, Fractional n, Real n) => Arbitrary (Duration n) where
   arbitrary = (fromRational . abs) <$> arbitrary
 
 instance Arbitrary a => Arbitrary (Dynamic a) where
@@ -71,35 +79,35 @@ instance Arbitrary a => Arbitrary (Active a) where
 
 instance Show a => Show (Active a) where
   show = onActive (\c -> "<<" ++ show c ++ ">>")
-                  (\d -> show d)
+                  show
 
-prop_era_start :: Time -> Time -> Bool
+prop_era_start :: Time Rational -> Time Rational -> Bool
 prop_era_start t1 t2 = start (mkEra t1 t2) == t1
 
-prop_era_end :: Time -> Time -> Bool
+prop_era_end :: Time Rational -> Time Rational -> Bool
 prop_era_end t1 t2 = end (mkEra t1 t2) == t2
 
-prop_duration :: Time -> Time -> Bool
+prop_duration :: Time Rational -> Time Rational -> Bool
 prop_duration t1 t2 = duration (mkEra t1 t2) == (t2 .-. t1)
 
-prop_shiftDynamic_start :: Duration -> Dynamic Bool -> Bool
+prop_shiftDynamic_start :: Duration Rational -> Dynamic Bool -> Bool
 prop_shiftDynamic_start dur dyn
   = (start . era) (shiftDynamic dur dyn) == ((start . era) dyn .+^ dur)
 
-prop_shiftDynamic_end :: Duration -> Dynamic Bool -> Bool
+prop_shiftDynamic_end :: Duration Rational -> Dynamic Bool -> Bool
 prop_shiftDynamic_end dur dyn
   = (end . era) (shiftDynamic dur dyn) == ((end . era) dyn .+^ dur)
 
-prop_shiftDynamic_fun :: Duration -> Dynamic Bool -> Time -> Bool
+prop_shiftDynamic_fun :: Duration Rational -> Dynamic Bool -> Time Rational -> Bool
 prop_shiftDynamic_fun dur dyn t
   = runDynamic dyn t == runDynamic (shiftDynamic dur dyn) (t .+^ dur)
 
-prop_active_semi_hom :: Active Any -> Active Any -> Time -> Bool
+prop_active_semi_hom :: Active Any -> Active Any -> Time Rational -> Bool
 prop_active_semi_hom a1 a2 t =
   runActive a1 t <> runActive a2 t == runActive (a1 <> a2) t
 
-prop_ui_id :: Time -> Bool
-prop_ui_id t = runActive (ui :: Active Time) t == t
+prop_ui_id :: Time Rational -> Bool
+prop_ui_id t = runActive (ui :: Active (Time Rational)) t == t
 
 prop_stretch_start :: Rational -> Active Bool -> Bool
 prop_stretch_start r a
@@ -115,7 +123,7 @@ prop_stretch_fun r (Blind a) t
   = runActive a t    runActive (stretch r t)
 -}
 
-prop_stretchTo_dur :: Positive Duration -> Active Bool -> Property
+prop_stretchTo_dur :: Positive (Duration Rational) -> Active Bool -> Property
 prop_stretchTo_dur (Positive dur) a
   = isDynamic a && ((duration <$> activeEra a) /= Just 0)
     ==> (duration <$> activeEra (stretchTo dur a)) == Just dur
@@ -136,20 +144,20 @@ prop_during_end d1 d2 =
  where a1 = fromDynamic d1
        a2 = fromDynamic d2
 
-prop_shift_start :: Duration -> Active Bool -> Bool
+prop_shift_start :: Duration Rational -> Active Bool -> Bool
 prop_shift_start d a =
   ((.+^ d) . start <$> activeEra a) == (start <$> activeEra (shift d a))
 
-prop_shift_end :: Duration -> Active Bool -> Bool
+prop_shift_end :: Duration Rational -> Active Bool -> Bool
 prop_shift_end d a =
   ((.+^ d) . end <$> activeEra a) == (end <$> activeEra (shift d a))
 
-prop_atTime_start :: Time -> Dynamic Bool -> Bool
+prop_atTime_start :: Time Rational -> Dynamic Bool -> Bool
 prop_atTime_start t dyn =
     (start <$> activeEra (atTime t a)) == Just t
   where a = fromDynamic dyn
 
-prop_atTime_fun :: Time -> Dynamic Bool -> Duration -> Bool
+prop_atTime_fun :: Time Rational -> Dynamic Bool -> Duration Rational -> Bool
 prop_atTime_fun t dyn d =
     runActive (atTime t a) (t .+^ d) == runActive a (s .+^ d)
   where a = fromDynamic dyn
